from manga_py.provider import Provider
from .helpers.std import Std


class PururinIo(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_main_content(self):
        url = self.get_url()
        if ~url.find('/gallery/'):
            re = r'/gallery/(\d+)/([^/]+)'
        else:
            re = r'/read/(\d+)/\d+/([^/]+)'
        return self._get_content('{}/gallery/{}/{}'.format(
            self.domain,
            *self.re.search(re, url).groups()
        ))

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/gallery/'):
            re = r'/gallery/\d+/([^/]+)'
        else:
            re = r'/read/\d+/\d+/([^/]+)'
        return self._get_name(re)

    def get_chapters(self):
        return [b'']

    def _images(self, content):
        items = self.json.loads(content)
        images = []
        for i in items:
            url = items.get(i, {}).get('image', False)
            url and images.append(url)
        return images

    def get_files(self):
        items = self._elements('.col-md-10 .well-pururin > div[class*="preview"] > a')
        if items:
            url = self.http().normalize_uri(items[0].get('href'))
            content = self.http_get(url)
            images = self.re.search(r'chapters\s*=\s*(\{.+\})\s*;', content)
            if images:
                return self._images(images.group(1))
        return []

    def get_cover(self) -> str:
        return self._cover_from_content('.cover > a > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PururinIo



from .read_powermanga_org import ReadPowerMangaOrg


class HatigarmScansEu(ReadPowerMangaOrg):
    _name_re = '/hs/[^/]+/([^/]+)'
    _content_str = '{}/hs/series/{}/'
    _chapters_selector = '.list-group-item .title a'


main = HatigarmScansEu



from manga_py.provider import Provider
from .helpers import animextremist_com
from .helpers.std import Std


class AnimeXtremistCom(Provider, Std):
    helper = None
    prefix = '/mangas-online/'

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = self.re.search(r'(.+?-\d+)', chapter[0])
        return idx.group(1) if idx else '0'

    def get_main_content(self):
        return self._get_content('{}%s{}/' % self.prefix)

    def get_manga_name(self) -> str:
        return self._get_name(r'{}([^/]+)'.format(self.prefix))

    def get_chapters(self):
        ch = self.helper.get_chapters()
        return ch[::-1]

    def get_files(self):
        chapter = self.chapter
        items = self.helper.sort_images(chapter[1])
        images = []
        for i in items:
            img = self.helper.get_page_image(i, 'img#photo')
            img and images.append(img)
        return images

    def prepare_cookies(self):
        self.helper = animextremist_com.AnimeXtremistCom(self)

    def get_cover(self) -> str:
        pass
        # return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = AnimeXtremistCom



import json
import sys
from argparse import ArgumentParser
from os import name as os_name

import requests
from packaging import version
from progressbar import ProgressBar

from .meta import __version__, __repo_name__
from .parser import Parser


def _image_args(args_parser):
    args = args_parser.add_argument_group('Image options')

    args.add_argument('--not-change-files-extension', action='store_const',
                      help='Save files to archive "as is"', const=True, default=False)

    # args.add_argument('--force-png', action='store_const', 
    #                          help='Force conversation images to png format', const=True, default=False)
    # args.add_argument('--force-jpg', action='store_const', 
    #                          help='Force conversation images to jpg format', const=True, default=False)

    # args.add_argument('-xt', type=int, help='Manual image crop with top side', default=0)
    # args.add_argument('-xr', type=int, help='Manual image crop with right side', default=0)
    # args.add_argument('-xb', type=int, help='Manual image crop with bottom side', default=0)
    # args.add_argument('-xl', type=int, help='Manual image crop with left side', default=0)
    # args.add_argument('--crop-blank', action='store_const', help='Crop white lines on image',
    #                          const=True, default=False)


def _debug_args(args_parser):
    args = args_parser.add_argument_group('Debug / Simulation options')

    args.add_argument('-h', '--help', action='help', help='show help and exit')
    args.add_argument('--print-json', action='store_const', const=True, default=False,
                      help='Print information about the results in the form of json (after completion)' +
                      ' - Not worked now')

    args.add_argument('--simulate', action='store_const', const=True, default=False,
                      help='Do not download the files and do not write anything to disk')

    args.add_argument('--full-error', action='store_const', const=True, default=False,
                      help='Show full stack trace')

    # args.add_argument('-vv', '--log', metavar='info', type='str', help='Verbose log')


def _downloading_args(args_parser):
    args = args_parser.add_argument_group('Downloading options')

    args.add_argument('-s', '--skip-volumes', metavar='count', type=int,
                      help='Skip volumes', default=0)
    args.add_argument('-c', '--max-volumes', metavar='count', type=int,
                      help='Maximum volumes for downloading 0=All', default=0)
    args.add_argument('--user-agent', type=str, help='Don\'t work from protected sites')
    args.add_argument('--proxy', type=str, help='Http proxy')
    args.add_argument('--reverse-downloading', action='store_const',
                      help='Reverse volumes downloading', const=True, default=False)
    args.add_argument('--rewrite-exists-archives', action='store_const', const=True,
                      default=False)
    args.add_argument('-nm', '--no-multi-threads', action='store_const',
                      help='Disallow multi-threads images downloading', const=True, default=False)
    args.add_argument('--zero-fill', action='store_const', const=True, default=False,
                      help='Adds 0 to the end for all chapters (vol_001.zip -> vol_001-0.zip)')


def _reader_args(args_parser):
    args = args_parser.add_argument_group('Archive options')

    args.add_argument('--cbz', action='store_const', default=False,
                      const=True, help='Make *.cbz archives (for reader)')

    args.add_argument('--rename-pages', action='store_const', default=False, const=True,
                      help='Normalize images names. (example: 0_page_1.jpg -> 0001.jpg)')


def get_cli_arguments() -> ArgumentParser:  # pragma: no cover
    args_parser = ArgumentParser(add_help=False)
    args = args_parser.add_argument_group('General options')

    args.add_argument('url', metavar='url', type=str, help='Downloaded url')
    args.add_argument('--version', action='version', version=__version__)

    args.add_argument('-n', '--name', metavar='name', type=str, help='Manga name', default='')
    args.add_argument('-d', '--destination', metavar='destination', type=str,
                      help='Destination folder (Default = current directory')
    args.add_argument('-np', '--no-progress', metavar='no-progress', action='store_const',
                      const=True, help='Don\'t show progress bar', default=False)
    # future
    # args_parser.add_argument('--server', action='store_const', const=True, help='Run web interface',
    #                          default=False)

    _image_args(args_parser)
    _reader_args(args_parser)
    _downloading_args(args_parser)
    _debug_args(args_parser)

    return args_parser


def check_version():
    api_url = 'https://api.github.com/repos/%s/releases/latest' % __repo_name__
    api_content = json.loads(requests.get(api_url).text)
    tag_name = api_content['tag_name']
    if version.parse(tag_name) > version.parse(__version__):
        download_addr = api_content['assets']
        if len(download_addr):
            url = download_addr[0]['browser_download_url']
        else:
            url = api_content['html_url']
        return {'message': 'Found new version', 'tag': tag_name, 'url': url, 'need_update': True}
    return {'message': 'Ok', 'need_update': False, 'tag': '', 'url': ''}


class Cli:
    args = None
    parser = None
    _info = None
    __progress_bar = None

    def __init__(self, args: ArgumentParser, info=None):
        self.args = args.parse_args()
        self.parser = Parser(args)
        self._info = info

    def start(self):
        self.parser.init_provider(
            progress=self.progress,
            log=self.print,
            quest=self.quest,
            info=self._info,
        )
        self.parser.start()
        self.__progress_bar and self.__progress_bar.value > 0 and self.__progress_bar.finish()
        self.print(' ')

    def input(self, prompt: str = ''):
        return input(prompt=prompt + '\n')

    def __init_progress(self, items_count: int, re_init: bool):
        if re_init or not self.__progress_bar:
            if re_init:
                print(' ')
                self.__progress_bar.finish()
            bar = ProgressBar()
            self.__progress_bar = bar(range(items_count))
            self.__progress_bar.init()

    def progress(self, items_count: int, current_item: int, re_init: bool = False):
        if not items_count:
            return
        if not self.args.no_progress and not self.args.print_json:
            current_val = 0
            if self.__progress_bar:
                current_val = self.__progress_bar.value
            self.__init_progress(items_count, re_init and current_val > 0)
            self.__progress_bar.update(current_item)

    def print(self, text, end='\n'):
        if os_name == 'nt':
            text = str(text).encode().decode(sys.stdout.encoding, 'ignore')
        print(text, end=end)

    def _single_quest(self, variants, title):
        self.print(title)
        for v in variants:
            self.print(v)
        return self.input()

    def _multiple_quest(self, variants, title):
        self.print('Accept - blank line + enter')
        self.print(title)
        for v in variants:
            self.print(v)
        result = []
        while True:
            _ = self.input().strip()
            if not len(_):
                return result
            result.append(_)

    def quest(self, variants: enumerate, title: str, select_type=0):  # 0 = single, 1 = multiple
        if select_type:
            return self._multiple_quest(variants, title)
        return self._single_quest(variants, title)



from manga_py.provider import Provider
from .helpers.std import Std


class MangaTailCom(Provider, Std):
    __local_storage = None

    def get_archive_name(self) -> str:
        return self.normal_arc_name([
            self.chapter_id,
            self.get_chapter_index()
        ])

    def get_chapter_index(self) -> str:  # Oh ...
        idx = self.chapter[0]
        re = self.re.search(r'.+\s(\d+(?:\.\d+)?)', idx)
        if re:
            return re.group(1)
        return idx

    def get_main_content(self):
        url = self.get_url()
        if self.__local_storage:
            url = self.__local_storage
        return self.http_get(url)

    def get_manga_name(self) -> str:
        selector = '.main-content-inner .page-header'
        header = self.html_fromstring(self.get_url(), selector, 0)
        link = header.cssselect('a.active + a')
        if link:
            link = self.http().normalize_uri(link.get('href'))
            self.__local_storage = link
            header = self.html_fromstring(link, selector, 0)
        return header.text_content().strip().replace('/', '_')  # http://www.mangasail.com/content/12-prince-manga

    def get_chapters(self):
        items = self.document_fromstring(self.content, '.chlist td a')
        n = self.http().normalize_uri
        return [(i.text_content().strip(), n(i.get('href'))) for i in items]

    def prepare_cookies(self):
        self._base_cookies()

    def get_files(self):
        url = self.chapter[1]
        items = self.html_fromstring('{}{}'.format(url, '?page=all'), '#images img')
        n = self.http().normalize_uri
        return [n(i.get('src')) for i in items]

    def get_cover(self) -> str:  # TODO
        cover = self.document_fromstring(self.content, 'iframe.authcache-esi-assembly', 0)
        cover = self.json.loads(cover.text_content().strip()).get('field')
        key = cover.keys()[0]
        cover = self.document_fromstring(cover.get(key), '.field-type-image img', 0)
        return self.http().normalize_uri(cover.get('src'))

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaTailCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaGoMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        tp = self.re.search('/(mf|raw)', self.chapter).group(1)
        return 'vol_{:0>3}-{}-{}'.format(*idx, tp)

    def get_chapter_index(self) -> str:
        selector = r'/(?:mf|raw)/.*?(\d+)(?:\.(\d+))?'
        chapter = self.chapter
        idx = self.re.search(selector, chapter).groups()
        return '{}-{}'.format(*self._idx_to_x2(idx))

    def get_main_content(self):
        return self._get_content('{}/read-manga/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'/read-manga/([^/]+)/')

    def get_chapters(self):
        content = self.document_fromstring(self.content, '#information', 0)
        chapters = content.cssselect('#chapter_table a.chico')
        raws = content.cssselect('#raws_table a.chicor')
        return chapters + raws

    def prepare_cookies(self):
        self.cf_protect(self.get_url())

    def get_files(self):
        content = self.http_get(self.chapter)
        parser = self.re.search("imgsrcs.+[^.]+?var.+?=\s?'(.+)'", content)
        if not parser:
            return []
        return parser.group(0).split(',')

    def get_cover(self):
        return self._cover_from_content('#information .cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaGoMe



from manga_py.provider import Provider
from .helpers.std import Std


class LusciousNet(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self.get_chapter_index())

    def get_main_content(self):
        name = self._get_name('/albums?/([^/]+)/')
        return self.http_get('{}/albums/{}/'.format(self.domain, name))

    def get_manga_name(self) -> str:
        return self._get_name('/albums?/([^/]+)_\d+/')

    def get_chapters(self):
        return [b'']

    def get_files(self):
        url = self.chapter
        items = self.html_fromstring(url, '#album_meta_ds .item > a')
        n = self.http().normalize_uri
        images = []
        for i in items:
            content = self.http_get(n(i.get('href')), headers={'x-requested-with': 'XMLHttpRequest'})
            image = self.document_fromstring(content, '#single_picture')
            if image:
                images.append(n(image[0].get('src')))
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.album_cover_item img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = LusciousNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaFoxMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        selector = '/manga/[^/]+/([^/]+)/'
        chapter = self.chapter
        idx = self.re.search(selector, chapter).group(1).split('.')
        return '{}-{}'.format(*self._idx_to_x2(idx))

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)/?')

    def get_chapters(self):
        return self._elements('#chapters a.tips')

    def __get_files_url(self):
        volume = self.chapter
        url = self.http().normalize_uri(volume)
        if ~url.find('.html'):
            url = url[: url.rfind('/')]
        return url

    def get_files(self):
        img_selector = 'img#image'
        _url = self.__get_files_url()
        url = '{}/1.html'.format(_url)
        selector = '#top_bar .r .l select.m option'
        parser = self.html_fromstring(url)
        pages = [i.get('value') for i in parser.cssselect(selector)]

        images = self._images_helper(parser, img_selector)

        for n in pages:
            if int(n) < 2:
                continue
            url = '{}/{}.html'.format(_url, n)
            parser = self.html_fromstring(url)
            images += self._images_helper(parser, img_selector)

        return images

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaFoxMe



from manga_py.provider import Provider
from .helpers.std import Std


class MangaOnlineToday(Provider, Std):
    _img_selector = '#sct_content img'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('.')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'\.today/[^/]+/([^/]+)', self.chapter)
        return idx.group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.today/([^/]+)')

    def get_chapters(self):
        return self._elements('ul.chp_lst a')

    def _pages_helper(self, options):
        images = []
        chapter = self.chapter
        for n in range(1, int(options)):
            content = self.html_fromstring('{}{}/'.format(chapter, n * 2 + 1))
            img = content.cssselect(self._img_selector)
            for i in img:
                images.append(i.get('src'))
        return images

    def get_files(self):
        images = []
        content = self.html_fromstring(self.chapter)
        img = content.cssselect(self._img_selector)
        if img:
            images = [i.get('src') for i in img]

        options = len(content.cssselect('.cbo_wpm_pag')[0].cssselect('option')) / 2 + .5
        return images + self._pages_helper(options)

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaOnlineToday



from manga_py.crypt import KissMangaComCrypt
from manga_py.fs import basename
from manga_py.provider import Provider
from .helpers.std import Std


class KissMangaCom(Provider, Std):
    __local_data = {
        'iv': b'a5e8e2e9c2721be0a84ad660c472c1f3',
        'key': b'mshsdf832nsdbash20asdm',
    }

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return 'Ch-{:0>3}_Vol-{:0>3}-{:0>1}'.format(*idx.split('-'))

    def get_chapter_index(self) -> str:
        bn = basename(self.chapter)
        name = self.re.search(r'Vol-+(\d+)-+Ch\w*?-+(\d+)-+(\d+)', bn)
        if name:
            name = name.groups()
            return '{1}-{0}-{2}'.format(*name)
        name = self.re.search(r'Vol-+(\d+)-+Ch\w*?-+(\d+)', bn)
        if name:
            name = name.groups()
            return '{1}-{0}-0'.format(*name)
        name = self.re.search(r'Ch\w*?-(\d+)(?:-v(\d+))?', bn).groups()
        return '{}-{}-0'.format(*name, '0')

    def get_main_content(self):
        return self._get_content('{}/Manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/Manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.listing td a')

    def prepare_cookies(self):
        self.cf_protect(self.get_url())
        self._storage['cookies']['rco_quality'] = 'hq'

    def __decrypt_images(self, crypt, key, hexes):
        images = []
        for i in hexes:
            img = crypt.decrypt(self.__local_data['iv'], key, i)
            images.append(img)

        return images

    def __check_key(self, crypt, content):
        # if need change key
        need = self.re.search(r'\["([^"]+)"\].+chko.?=.?chko', content)
        key = self.__local_data['key']
        if need:
            key += crypt.decode_escape(need.group(1))
        else:
            # if need change key
            need = self.re.search(r'\["([^"]+)"\].*?chko.*?=.*?chko', content)
            if need:
                key = crypt.decode_escape(need.group(1))
        return key

    def get_files(self):
        crypt = KissMangaComCrypt()

        content = self.http_get(self.chapter)
        key = self.__check_key(crypt, content)

        hexes = self.re.findall(r'lstImages.push\(wrapKA\(["\']([^"\']+?)["\']\)', content)

        if not hexes:
            return []

        images = self.__decrypt_images(crypt, key, hexes)

        self._storage['referer'] = self.http().referer = ''
        return [i.decode('utf-8').replace('\x10', '').replace('\x0f', '') for i in images]

    def get_cover(self):
        return self._cover_from_content('.rightBox .barContent img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = KissMangaCom



from manga_py.provider import Provider
from .helpers.std import Std


class DanbooruDonmaiUs(Provider, Std):
    _is_tag = False
    _archive_prefix = 'danbooru_'
    _manga_name = None

    def get_archive_name(self) -> str:
        if self.chapter:
            return 'page_{:0>2}'.format(self.chapter)
        return 'archive'

    def get_chapter_index(self) -> str:
        if self.chapter:
            return str(self.chapter)
        return '0'

    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        if ~self.get_url().find('tags='):
            self._is_tag = True
            self._manga_name = self._get_name(r'[\?&]tags=([^&]+)')
        else:
            self._manga_name = self._get_name(r'/posts/(\d+)')
        return self._archive_prefix + self._manga_name

    def get_chapters(self):  # pragma: no cover
        if self._is_tag:
            pages = self._elements('.paginator .current-page > span')
            images_on_page = len(self._elements('#posts > div > article'))
            if pages:
                count = self.html_fromstring('{}/counts/posts?tags={}'.format(
                    self.domain,
                    self.manga_name,
                ), '#a-posts', 0).text_content()
                page = self.re.search(r'\n\s+(\d+)', count).group(1)
                max_page = int(int(page) / images_on_page) + 1
                if max_page > 1001:
                    self.log('1000 pages maximum!')
                    max_page = 1000
                return range(1, max_page)[::-1]
        return [1]

    def _tag_images(self):  # pragma: no cover
        url = '{}/posts?tags={}&page={}'.format(
            self.domain,
            self._manga_name,
            self.chapter,
        )
        parser = self.html_fromstring(url, '#posts article a')
        n = self.http().normalize_uri
        images = []
        for i in parser:
            images += self._post_image(n(i.get('href')))
        return images

    def _post_image(self, url):  # pragma: no cover
        if isinstance(url, str):
            parser = self.html_fromstring(url)
        else:
            parser = url

        full_size = parser.cssselect('#image-resize-notice a')
        if full_size:
            return [full_size[0].get('href')]
        return [parser.cssselect('#image')[0].get('src')]

    def _post_images(self, url):  # pragma: no cover
        parser = self.html_fromstring(url)
        links = parser.cssselect('#has-parent-relationship-preview article a')
        if links:
            images = []
            n = self.http().normalize_uri
            for i in links:
                images += self._post_image(n(i.get('href')))
            return images
        return self._post_image(parser)

    def get_files(self):
        if self._is_tag:
            return self._tag_images()
        return self._post_images(self.get_url())

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = DanbooruDonmaiUs



try:
    from atexit import register as atexit_register
    from os import makedirs
    from os import path
    from shutil import rmtree
    from sys import argv, exit, exc_info, stderr
    from json import dumps
    import traceback

    from .cli import Cli, get_cli_arguments, check_version
    from .fs import get_temp_path
    from .info import Info

except Exception as e:
    print(e)
    print('Setup in progress?')
    exit()

__author__ = 'Sergey Zharkov'
__license__ = 'MIT'
__email__ = 'sttv-pc@mail.ru'


@atexit_register
def before_shutdown():
    temp_dir = get_temp_path()
    path.isdir(temp_dir) and rmtree(temp_dir)


def _init_cli(args, _info):
    error_lvl = -2
    if args.parse_args().full_error:
        error_lvl = -10
    try:
        _info.start()
        cli_mode = Cli(args, _info)
        cli_mode.start()
        code = 0
    except Exception as e:
        traceback.print_tb(e.__traceback__, error_lvl, file=stderr)
        code = 1
        _info.set_error(e)
    return code


def main():
    temp_path = get_temp_path()
    path.isdir(temp_path) or makedirs(temp_path)

    args = get_cli_arguments()
    parse_args = args.parse_args()

    # if parse_args.server:
    #     server_mode = Server(args)
    #     exit()

    _info = Info(parse_args)

    code = _init_cli(args, _info)

    if parse_args.print_json:
        print(dumps(_info.get()))

    exit(code)


if __name__ == '__main__':
    # version = check_version()  # maybe
    # if version['need_update']:
    #     print('Found new version! %s \nSee here: %s' % (
    #         version['tag'],
    #         version['url']
    #     ), file=stderr)

    main()



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicBooksOnlineOrg(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/reader/[^/]+_(\d+(?:/\d+)?)')
        idx = re.search(self.chapter).groups()
        return '-'.join(idx.split('/'))

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.(?:org|net)/(?:reader/)?([^/]+)')

    def get_chapters(self):
        s = '#chapterlist .chapter > a'
        return self.document_fromstring(self.content, s)

    def _get_image(self, parser):
        src = parser.cssselect('a > img.picture')
        if not src:
            return None
        return '{}/reader/{}'.format(self.domain, src[0].get('src'))

    def get_files(self):
        chapter = self.chapter
        content = self.html_fromstring(chapter, '.pager select[name="page"]', 0)
        pages = [i.get('value') for i in content.cssselect('option + option')]
        img = self._get_image(content)
        images = []
        img and images.append(img)
        for i in pages:
            _content = self.html_fromstring('{}/{}'.format(chapter, i))
            img = self._get_image(_content)
            img and images.append(img)
        return images

    def get_cover(self):
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadComicBooksOnlineOrg



from manga_py.fs import dirname, path_join, get_temp_path, rename
from manga_py.provider import Provider
from .helpers.std import Std


class MangaChanMe(Provider, Std):
    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        name = r'\.me/[^/]+/\d+-(.+)\.html'
        return self._get_name(name)

    def loop_chapters(self):
        arc_name = self.get_archive_name()
        arc = self._archive_type()
        path = path_join(dirname(self.get_archive_path()), arc_name + '.%s' % arc)
        url = self.chapter
        temp_path = get_temp_path('{:0>2}_{}-temp_arc.zip'.format(
            self._storage['current_chapter'],
            arc_name
        ))
        self.save_file(url, temp_path)
        rename(temp_path, path)

    def get_chapters(self):
        selector = r'\.me/[^/]+/(\d+-.+\.html)'
        url = self._get_name(selector)
        url = '/download/{}'.format(url)
        return self.html_fromstring(url, 'table#download_table tr td + td > a')

    def get_files(self):
        return []

    def get_cover(self):
        selector = r'\.me/[^/]+/(\d+-.+\.html)'
        url = self._get_name(selector)
        url = '{}/manga/{}'.format(self.domain, url)
        img = self._elements('#cover', self.http_get(url))
        if img and len(img):
            return img[0].get('src')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaChanMe



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHubIo(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(r'/chapter/[^/]+/\w+-([^/]+)', chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)')

    def get_chapters(self):
        return self._elements('.list-group .list-group-item > a')

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self._elements('#mangareader img[src*="/file/"]', content)
        n = self.http().normalize_uri
        return [n(i.get('src')) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.row > div > img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHubIo



import cfscrape


class CloudFlareProtect:
    protector = []

    def run(self, url):  # pragma: no cover

        if not self.protector:
            scr = cfscrape.create_scraper()
            try:
                self.protector = scr.get_tokens(url)
            except Exception:
                pass

        return self.protector



from manga_py.provider import Provider
from .helpers.std import Std


class SiberOwlCom(Provider, Std):
    _main_fmt = '{}/mangas/{}/'
    n = None

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('.', 2)
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/mangas/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content(self._main_fmt)

    def get_manga_name(self) -> str:
        return self._get_name(r'/mangas/([^/]+)')

    def get_chapters(self):
        re = self.re.compile(r'chapString\s*=\s*"(.+)";')
        elements = self.document_fromstring(
            re.search(self.content).group(1),
            'a'
        )
        return ['{}/mangas/{}/{}'.format(
            self.domain,
            self.manga_name,
            i.get('href')
        ) for i in elements]

    def get_files(self):
        content = self.http_get(self.chapter)
        re = self.re.search(r'imageUrls\s*=\s*(\[.*\])', content)
        items = re.group(1)
        if not items:
            return []
        items = self.json.loads(self.re.sub(r'(.+)",\]', r'\1"]', items))
        return ['{}{}'.format(self.domain, i) for i in items]

    def get_cover(self) -> str:
        re = self.re.compile(r'imageUrl\s*=\s*"(.+)";')
        return '{}{}'.format(
            self.domain,
            re.search(self.content).group(1)
        )

    def book_meta(self) -> dict:
        # todo meta
        pass


main = SiberOwlCom



from abc import abstractmethod


class Abstract:

    @abstractmethod
    def get_main_content(self):  # call once
        pass

    @abstractmethod
    def get_manga_name(self) -> str:  # call once
        return ''

    @abstractmethod
    def get_chapters(self) -> list:  # call once
        return []

    def prepare_cookies(self):  # if site with cookie protect
        pass

    @abstractmethod
    def get_files(self) -> list:  # call ever volume loop
        return []

    @abstractmethod
    def get_archive_name(self) -> str:
        pass

    #  for chapters selected by manual (cli)
    @abstractmethod
    def get_chapter_index(self) -> str:
        pass

    def book_meta(self) -> dict:
        pass

    def loop_callback_chapters(self):
        pass

    def get_cover(self):
        pass

    def before_file_save(self, url, idx) -> str:  # return url !
        return url

    def after_file_save(self, _path: str, idx: int):
        pass



from typing import Callable


class Callbacks:
    def _call_files_progress_callback(self):
        if callable(self.progress):
            _max, _current = len(self._storage['files']), self._storage['current_file']
            self.progress(_max, _current, _current < 1)

    def set_quest_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        setattr(self, 'quest', callback)

    def set_progress_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        setattr(self, 'progress', callback)

    def set_log_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        setattr(self, 'log', callback)

    def quest(self, *args, **kwargs):
        pass

    def progress(self, *args, **kwargs):
        pass

    def log(self, *args, **kwargs):
        pass

    def book_meta(self) -> dict:
        return {}



from manga_py.provider import Provider
from .helpers.std import Std


class MangaEdenCom(Provider, Std):
    uriRegex = r'/[^/]+/([^/]+-manga)/([^/]+)/?'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'-manga/[^/]+/(\d+)', self.chapter).group(1)
        return '{}-0'.format(idx)

    def get_main_content(self):
        result = self.re.search(self.uriRegex, self.get_url())
        groups = result.groups()
        return self.http_get('{}/en/{}/{}/'.format(self.domain, *groups))

    def get_manga_name(self) -> str:
        return self.re.search(self.uriRegex, self.get_url()).group(2)

    def get_chapters(self):
        return self._elements('a.chapterLink')

    def get_files(self):
        content = self.http_get(self.chapter)
        result = self.re.search(r'var\s+pages\s+=\s+(\[{.+}\])', content)
        items = []
        if not result:
            return []
        for i in self.json.loads(result.group(1)):
            items.append('http:' + i['fs'])
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('#rightContent .info img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaEdenCom



from .read_powermanga_org import ReadPowerMangaOrg


class Pzykosis666HFansubCom(ReadPowerMangaOrg):
    _name_re = '/online/[^/]+/([^/]+)/'
    _content_str = '{}/online/series/{}/'


main = Pzykosis666HFansubCom



from manga_py.provider import Provider
from .helpers.std import Std


class _Template(Provider, Std):

    def get_archive_name(self) -> str:
        pass

    def get_chapter_index(self) -> str:
        pass

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        return ''

    def get_chapters(self):
        # return self._elements('a.chapter')
        return []

    def get_files(self):
        return []

    def get_cover(self) -> str:
        # return self._cover_from_content('.cover img')
        pass

    def book_meta(self) -> dict:
        """
        :see http://acbf.wikia.com/wiki/Meta-data_Section_Definition
        return {
            'author': str,
            'title': str,
            'annotation': str,
            'keywords': str,
            'cover': str,
            'rating': str,
          }
        """
        pass


main = _Template
